# inspired from:
# https://github.com/pypa/warehouse/blob/master/warehouse/__about__.py

__all__ = [
    '__title__', '__summary__', '__uri__', '__version__',
    '__author__', '__email__', '__license__', '__copyright__',
]

__title__ = 'sagalogger'
__summary__ = 'A json log formatter.'
__uri__ = 'https://www.github.com/Sagacify/logger-py'

__version__ = '0.3.2'


__author__ = 'Sagacify'
__email__ = 'dev+python@sagacify.com'

__license__ = 'MIT'
__copyright__ = '2016 %s' % __author__
